
/* NFuncs.h
 * SBC++ (soccer client for Robocup2000)
 * Eslam Nazemi<nazemi@cc.sbu.ac.ir>
 * Amin Abbaspour<a-abbaspour@ce.sbu.ac.ir>
 * Mahmood Rahmani<m-rahmani@ce.sbu.ac.ir>
 * Bahman Radjabalipoor<b-radjabalipoor@ce.sbu.ac.ir>
 * Computer Department
 * Shahid Beheshti University
*/


#ifndef __NETWORK_FUNCTIONS_H
#define __NETWORK_FUNCTIONS_H

#include "NetHeader.h"

typedef float (*FuncType)();
typedef void  (*BehaveType)();

typedef enum CommandList {
    nothing			,	/*	0	*/		
    ballahead			,	/*	1	*/
    ballbehind			,	/*	2	*/
    balliscatchable		,	/*	3	*/
    ballisfar			,	/*	4	*/
    ballisnearothergoal		,	/*	5	*/
    ballisnearowngoal		,	/*	6	*/
    ballisverynearowngoal	,	/*	7	*/
    ballkickable		,	/*	8	*/
    balloutside			,	/*	9	*/
    catchsuccess		,	/*	10	*/
    freeteammateisoffside	,	/*	11	*/
    haveenoughstamina		,	/*	12	*/
    highstamina			,	/*	13	*/ 
    inotherhalf			,	/*	14	*/
    justcaughtball		,	/*	15	*/
    knowmarkedopponent		,	/*	16	*/
    knownotinoffside		,	/*	17	*/
    lowstamina			,	/*	18	*/
    maysaysomething		,	/*	19	*/
    mygoalkick			,	/*	20	*/
    nearball			,	/*	21	*/
    neargoal			,	/*	22	*/
    nearopponent		,	/*	23	*/
    nearowngoal			,	/*	24	*/
    offsideposition		,	/*	25	*/
    opponentisfree 		,	/*	26  	*/
    opponentinfront		,	/*	27	*/	
    playmodeon			,	/*	28	*/
    standfree			,	/*	29	*/
    teammatebehind		,	/*	30	*/
    teammateinfront		,	/*	31	*/	
    teammateisbetweenball	,	/*	32	*/
    teammateisfree		,	/*	33	*/
    teammateisnearerball	,	/*	34	*/
    teammatesinplaces		,	/*	35	*/
    theirgoalkick		,	/*	36	*/
    
    haveball			, 	/*	37	*/
    havemarked			,	/*	38	*/
    haveseenball		,	/*	39	*/
    losestamina			,	/*	40	*/
    inoffside			,	/*	41	*/
    noothergoal			,	/*	42	*/
    offsidetrap			,	/*	43	*/
    soccergoal			,	/*	44	*/
    teammatehasball         		/*	45	*/
 };

#define MaxCommands		46
#define GoalCommandStart        36

typedef enum BehaveList  {
    nothing_beh			,	/*	0 	*/
    catchball			,	/*	1 	*/
    checkoffside		,	/*	2 	*/
    cover			,	/*	2.5	*/
    getopen			,	/* 	2.75	*/
    kickawayball		,	/*	3 	*/
    kickgoal			,	/*	4 	*/
    markopponent		,	/*	5	*/
    mindball			,	/*	6 	*/
    movetohomeposition		,	/*	7 	*/
    lookstraight		,	/*	8 	*/
    passball			,	/*	9 	*/
    pushbackward		,	/*	0 	*/
    pushforward			,	/*	11 	*/
    refreshfield		,	/*	12 	*/
    relax			,	/*	13 	*/
    retreat			,	/*	14 	*/
    runforwardtoball		,	/*	15 	*/
    runtoball			,	/*	16 	*/
    runtohomeposition		,	/*	17 	*/
    runwithball			,	/*	18 	*/
    sayposition			,	/*	19 	*/
    searchball			,	/*	20 	*/
    turnaroundballforkickin		/*	21 	*/
};

#define MaxBehaves		24

class TCommandsList
{
private:
    char	sName[STRLEN];
    Bool	bUsed;
    float	fOutput;

public:
    TCommandsList() {
	sName[0] = '\0';
	bUsed	 = FALSE;
	fOutput  = 0.0;
    }
    
    inline char *GetName() {return sName; } 
    inline float GetOutput() const { return fOutput; } 
    inline Bool IsUsed() const { return bUsed;}
    inline void SetUsed(const int& b) { bUsed = (Bool)b; }
    inline void SetOutput(const float& val) { fOutput = val; }
    inline void SetName(const char *s) { strcpy(sName, s); }
};
    
class TNetCommand
{
private:
    TCommandsList *Command[MaxCommands];
    FuncType	   FastFunc[MaxCommands];

    void InitFastFuncs();
    int Count;

public:
    TNetCommand();
    ~TNetCommand() { for(int i = 0; i < MaxCommands; i++) delete Command[i]; }
	    
    int   IsValid(const char *s);

    inline void  RunAllTruePercept() {
	for (Count = 0; Count < MaxCommands; Count++) {
	    if( Command[Count]->IsUsed() ) {
    		Command[Count]->SetOutput( (FastFunc)[Count]() );
    	    }
	    else
		Command[Count]->SetOutput(0.0);
	}
    }

    inline float GetFuncValue(const int& i, float val = 1.0) const {
	if( i < 0 ) return 1.0;
	else 
	if(val > 0) return Command[i]->GetOutput();
	else return (1.0 - Command[i]->GetOutput());    
    }

    inline int   GetFuncNum(const char *s) { return IsValid(s); }
};

class TNetBehave
{
    char  sBehaveList[MaxBehaves][STRLEN];
    BehaveType 	FastBehave[MaxBehaves];

    void  InitFastBehaves();
public:
    TNetBehave();

    int  IsValid(const char *s);
    void RunBehave(const int& i) const { (FastBehave)[i](); }
    inline int  GetBehaveNum(const char *s) { return IsValid(s); }
};

    float GetLinearFuzzyValue(const float& value, const float& left, 
			 	const float& right, const int& up = TRUE);

    Unum GetOpponentInWay(float dir);
    Unum GetFreeTeammate();
    
    float GetNothing();
    float GetBallAhead();
    float GetBallBehind();
    float GetBallIsCatchable();
    float GetBallIsFar();
    float GetBallIsNearOtherGoal();
    float GetBallIsNearOwnGoal();
    float GetBallIsVeryNearOwnGoal();
    float GetBallKickable();
    float GetBallOutside();
    float GetCatchSuccess();
    float GetFreeTeammateIsOffside();
    float GetHaveEnoughStamina();
    float GetHighStamina();
    float GetInOtherHalf();
    float GetJustCaughtBall();
    float GetKnowMarkedOpponent();
    float GetKnowNotInOffside();
    float GetLowStamina();
    float GetMaySaySomething();
    float GetMyGoalKick();
    float GetNearBall();
    float GetNearGoal();
    float GetNearOpponent();
    float GetNearOwnGoal();
    float GetOffsidePosition();
    float GetOpponentIsFree();
    float GetOpponentInFront();
    float GetPlayModeOn();
    float GetStandFree();
    float GetTeammateBehind();
    float GetTeammateInFront();
    float GetTeammateIsBetweenBall();
    float GetTeammateIsFree();
    float GetTeammateIsNearerBall();
    float GetTeammatesInPlaces();
        
    float GetHaveBall();
    float GetHaveMarked();
    float GetHaveSeenBall();
    float GetLoseStamina();
    float GetInOffside();
    float GetNoOtherGoal();
    float GetOffsideTrap();
    float GetSoccerGoal();
    float GetTeammateHasBall();
    float GetTheirGoalKick();
    
    void DoNothing();
    void DoCatchBall();
    void DoCheckOffside();
    void DoCover();
    void DoGetOpen();
    void DoKickAwayBall();
    void DoKickGoal();
    void DoLookStraight();
    void DoLookToBall();
    void DoMarkOpponent();
    void DoMindBall();
    void DoMoveToHomePosition();
    void DoPassBall();
    void DoPushBackward();
    void DoPushForward();
    void DoRefreshField();
    void DoRelax();
    void DoRetreat();
    void DoRunForwardToBall();
    void DoRunToBall();
    void DoRunToHomePosition();
    void DoRunWithBall();
    void DoSayPosition();
    void DoSearchBall();
    void DoTurnAroundBallForKickIn();

    void _DoNothing();
    void _DoCatchBall();
    void _DoCheckOffside();
    void _DoCover();
    void _DoGetOpen();
    void _DoKickAwayBall();
    void _DoKickGoal();
    void _DoLookStraight();
    void _DoLookToBall();
    void _DoMarkOpponent();
    void _DoMindBall();
    void _DoMoveToHomePosition();
    void _DoPassBall();
    void _DoPushBackward();
    void _DoPushForward();
    void _DoRefreshField();
    void _DoRelax();
    void _DoRetreat();
    void _DoRunForwardToBall();
    void _DoRunToBall();
    void _DoRunToHomePosition();
    void _DoRunWithBall();
    void _DoSayPosition();
    void _DoSearchBall();
    void _DoTurnAroundBallForKickIn();

#endif 
