
/* NetNodes.h
 * SBC++ (soccer client for Robocup2000)
 * Eslam Nazemi<nazemi@cc.sbu.ac.ir>
 * Amin Abbaspour<a-abbaspour@ce.sbu.ac.ir>
 * Mahmood Rahmani<m-rahmani@ce.sbu.ac.ir>
 * Bahman Radjabalipoor<b-radjabalipoor@ce.sbu.ac.ir>
 * Computer Department
 * Shahid Beheshti University
*/

#ifndef __NETWORK_NODES_H
#define __NETWORK_NODES_H

#include "NetHeader.h"
#include "NetFuncs.h"
#include "utils.h"
#include <stdio.h>

class TFunction
{
private:
    int     iFunc;
    float   fExp;

public:

    int     iComp;

    TFunction()
 {
	iFunc = 0;
    	fExp  = 0.0;
	iComp = 0;
    }

    inline void  SetFunc(const int& i) { iFunc = i; }
    inline void  SetExp (const float& val) { fExp = val; }

    inline int   GetFunc() const { return iFunc; }
    inline float GetExp() const  { return fExp; }
};

class TNetNode
{
private:
    char    sName[STRLEN];
    int     iFunc;
public:
    TNetNode() {
	sName[0] = '\0';
	iFunc    = 0;
    }
    ~TNetNode() { };

    inline char* GetName() {
 return sName;
 }
    inline int   GetFunc() const {
 return iFunc;
}
    inline void  SetName(const char *s) 
{
	char *s2 = strdup(s);
	down(s2);
	strcpy(sName, s2);
    }
    inline void  SetFunc(const int& i) {
 iFunc = i;
 }
};


class TCompetence : public TNetNode
{
private:
    Bool	bExecuted;

    float   	fAct;
    float 	fPrevAct;
    float 	fActOfGoal[MaxGoals];
    float	fPrevActOfGoal[MaxGoals];
    float   	fExec;
    float	fH;
    int	   	iResource;

    int     	iPreIndex;
    int     	iEffIndex;
    int    	iExpIndex;
    int		iGoalIndex;
    
    float	fGama;
    float	fDelata;

public:
    TFunction   *rPre[MaxPreConditions];
    TFunction   *rEff[MaxEffects];
    TFunction   *rExp[MaxExpantions];

    TCompetence();
    ~TCompetence();

    void  AddCondition(const int& i, const float& val = 1.0);
    void  AddEffect(const int& i, const float& val = 1.0);
    void  AddExpantion(const int& i, const int& j, const float& val = 1.0);
    void  SetResource(const char *res);

    inline void  SetAct(const float& val) { fAct = val; }
    inline void  SetActOfGoal(const int& i, const float& val) { 
	if( val > fActOfGoal[i] )
	    fActOfGoal[i] = val;
    }
    inline void  SetPrevAct(const float& val) { fPrevAct = val; }
    inline void  SetPrevActOfGoal(const int& i, const float& val) { fPrevActOfGoal[i] = val; }
    inline void  SetGama(const float& val) { fGama = val;}
    inline void  SetExec(const float& val) { fExec = val;}
    inline void  SetH(const float& val) { fH = val;}
    inline void  SetExecuted(const Bool& b) { bExecuted = b;}
    inline void  SetPreIndex(const int& i) { iPreIndex = i; }            
    inline void  SetEffIndex(const int& i) { iEffIndex = i; }            
    inline void  SetExpIndex(const int& i) { iExpIndex = i; }            
    inline void  SetGoalIndex(const int& i) { iGoalIndex = i; }            
    
    inline float GetAct() const { return fAct; }
    inline float GetActOfGoal(const int& i) const { 
	if(fActOfGoal[i] == -1000) 
	    return 0.0;
	else 
	    return fActOfGoal[i]; 
    }
    inline float GetPrevAct() const { return fPrevAct; }
    inline float GetPrevActOfGoal(const int& i) const { return fPrevActOfGoal[i]; }
    inline float GetGama() const { return fGama; }
    inline float GetExec() const { return fExec; }
    inline float GetH() const { return fH; }
    inline int 	 GetResource() const { return iResource; }
    inline int   GetPreIndex() const { return iPreIndex; }
    inline int   GetEffIndex() const { return iEffIndex; }
    inline int   GetExpIndex() const { return iExpIndex; }
    inline int   GetGoalIndex() const { return iGoalIndex; }
};


class TGoal : public TNetNode
{
private:
    float fImp;
    float fOutput;
	
public:
    TFunction  *Percept;

    TGoal() {
        Percept = new TFunction;
	if(!Percept) {
	    my_error("TGoal : Cann't allocate mem 4 Perceptron.\n");
	    _exit(0);
	}
	SetImp();
        SetOutput();
    }

    ~TGoal() { delete Percept; }


    inline  void  AddPercept(const int& i = 0, const float& val = 1.0) const {
        Percept->SetFunc(i);
	Percept->SetExp(val);
    }

    inline  float GetImp() const { return fImp; }
    inline  float GetOutput() const { 
	if( fImp > 0) return fOutput * fImp;
	else return - (1.0 - fOutput) * fImp;
    }
    
    inline  void  SetImp(const float& val = 1.0) { fImp = val;
 }
    inline  void  SetOutput(const float& val = 0.0) { fOutput = val; }
};

#endif // __NETWORK_NODES_H

