
/* ParsTools.C
 * SBC++ (soccer client for Robocup2000)
 * Eslam Nazemi<nazemi@cc.sbu.ac.ir>
 * Amin Abbaspour<a-abbaspour@ce.sbu.ac.ir>
 * Mahmood Rahmani<m-rahmani@ce.sbu.ac.ir>
 * Bahman Radjabalipoor<b-radjabalipoor@ce.sbu.ac.ir>
 * Computer Department
 * Shahid Beheshti University
*/

#include <stdio.h>
#include <string.h>

#include "NetHeader.h"
#include "ParsTools.h"
#include "utils.h"

TParseTools::TParseTools(const char *sFileName)
{	
    Initialize();
    PreParse(sFileName);
}

TParseTools::~TParseTools() { }

void TParseTools::CloseFile()
{
    fclose(in);
}

void TParseTools::Initialize()
{
    iCompBlockNum = 0;
    iGoalBlockNum = 0;
    iBlockNum     = 0;
}

void TParseTools::CleanParseString()
{
    for(int i = 0; i < ParseBlockMaxLength; i++) sParseString[i][0] = '\0';
}

void TParseTools::PreParse(const char *sFileName)
{
    char str[STRLEN];
    OpenFile(sFileName);
    fscanf(in, "%s", str);
    while( !feof(in) ) {
        if( !strcmp(str, STRING_IF) )   iCompBlockNum++;
        else if( !strcmp(str, STRING_GOAL) ) iGoalBlockNum++;
        fscanf(in, "%s", str);
    }
    fseek(in, 0, SEEK_SET);
}

Bool TParseTools::ReadNextBlock()
{
    int   i = 0;
    char  str[STRLEN];
    CleanParseString();

    iBlockNum++;

    fscanf(in, "%s", str);
    up(str);
    while( (strcmp(str, STRING_IF)  &&
	    strcmp(str, STRING_GOAL) ) && !feof(in)) {
        fscanf(in, "%s", str);
        up(str);
    }

    strcpy(sParseString[0], str);

    if(!strcmp(str, STRING_GOAL)) {
	while(strcmp(str, STRING_END_GOAL) && !feof(in)) {
    	    fscanf(in, "%s", str);
	    if( str[0] != '#' ) {
	        up(str);
	        strcpy(sParseString[++i], str);
	    }
	}
	if( strcmp(sParseString[i], STRING_END_GOAL))	return FALSE;
	else	return TRUE;
    }
    else 
	if(!strcmp(str, STRING_IF)) {
	    while(strcmp(str, STRING_END_IF) && !feof(in)) {
    		fscanf(in, "%s", str);
		if( str[0] != '#' ) {
		    up(str);
		    strcpy(sParseString[++i], str);
		}
	    }
	    DEBUG(printf("ReadNetxBlock : Finished reading a comp block. "));
	    if( strcmp(sParseString[i], STRING_END_IF)) return FALSE;
	    else return TRUE;
	}
	else return FALSE;
}


int  TParseTools::BlockType()
{
    if( !strcmp(sParseString[0], STRING_IF)) return BLOCK_COMPETENCE;
    else if( !strcmp(sParseString[0], STRING_GOAL)) return BLOCK_GOAL;
    else return BLOCK_UNKOWN;
}

void TParseTools::OpenFile(const char *sFileName)
{
    in = fopen(sFileName, "r");
    if(!in) {
	my_error("Unable to open rule file.\n");
	_exit(0);
    }
}

void TParseTools::ParseCompetence(TCompetence& newComp)
{
	int	    i = 0;
	int         e = 1;
	float       f = 1.0;

	while(strcmp(sParseString[++i], STRING_THEN)) {
    	    if( !strcmp(sParseString[i], STRING_AND)) {
		i++;
		if(!strcmp(sParseString[i], STRING_NOT)) {
		    i++;
		    e = -1;
		}
	    }
	    else 
		if(!strcmp(sParseString[i], STRING_NOT)) {
		    i++;
		    e = -1;
		}

	    newComp.AddCondition(GetFuncNum(sParseString[i]), e);
	    e = 1;
	}

	newComp.SetName(sParseString[++i]);
	newComp.SetFunc(GetBehaveNum(sParseString[i]));
    	i++;
	while( strcmp(sParseString[++i], STRING_USING) &&
	       strcmp(sParseString[i], STRING_END_IF) )	{
	    if( !strcmp( sParseString[i], STRING_AND)) {
		i++;
		if( !strcmp(sParseString[i], STRING_NOT) ) {
		    i++;
		    e = -1;
		}
	    }
	    else
		if(!strcmp(sParseString[i], STRING_NOT)) {
	    	    i++;
		    e = -1;
		}

	    if((f = atof(sParseString[i+1])) == 0) f = 1.0;
		newComp.AddEffect( GetFuncNum(sParseString[i]), e * f);

    	    e = 1;
	    f = 1.0;
	    i++;
	}
	
	if(!strcmp(sParseString[i], STRING_END_IF)) return;
	else if(!strcmp(sParseString[i], STRING_USING))
		newComp.SetResource(sParseString[i+1]);
}

void TParseTools::ParseGoal(TGoal& newGoal)
{
    int   e = 1;
    int   i = 0;
    
    if(!strcmp(sParseString[1], STRING_NOT)) {
	    e = -1;
	    i =  1;
	}    

    if(!strcmp(sParseString[i+2], STRING_THEN)) {

	newGoal.AddPercept( GetFuncNum(sParseString[i+1]), e );

	if(!strcmp(sParseString[i+3], STRING_NOT)) {
	    e = -1;
	    i++;
	}
	else e = 1;    

	newGoal.SetName( sParseString[i+3] );
    	newGoal.SetFunc( GetFuncNum(sParseString[i+3]) );
	newGoal.SetImp ( e * atof(sParseString[i+4]) );
	return;
    }
    else {
    	if( !strcmp(sParseString[1], STRING_NOT) ) {
	    e = -1;
	    i =  1;
	}
	newGoal.AddPercept( -1, 1.0 );
	newGoal.SetName( sParseString[i+1] );
	newGoal.SetFunc( GetFuncNum(sParseString[i+1]) );
	newGoal.SetImp ( e * atof(sParseString[i+2]) );
	return;
    }
}
